/* Generalized routines to perform simulated annealing on an arbitrary
   system with an arbitrary evaulation function and random step
   function.

   Created September 9th, 1998. 

   Copyright Nat Friedman, all rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "anneal.h"

int
metrop (long curr_energy_l, long new_energy_l, double temperature)
{
  double curr_energy, new_energy;

  new_energy = (double) new_energy_l / (double) ~0UL;
  curr_energy = (double) curr_energy_l / (double) ~0UL;
  
  if (new_energy < curr_energy)
    return 1;

  if (new_energy > curr_energy)
    {
      double r;
      /* Randomly decide whether or not to choose a system with higher
	 energy, with probability e^(- delta energy / temperature).
	 This is the Boltzmann probability distribution. */
      r = (double) rand() / (double) RAND_MAX;

      if (r < exp ((curr_energy - new_energy) / temperature))
	return 1;
    }

  return 0;
} /* metrop */

#if 0
void *
anneal(void * system, /* This is the system to be minimized */
       void step_func (void * system, void * dest_system),
       /* This is the function which generates a random step */
       long eval_func (void * system), /* This is the function which
				     determines the energy of a given
				     system. */
       void * temp_system)
     
{
  float temperature;
  long curr_energy, new_energy;
  void * new_system, * curr_system;
  int step_num;

  temperature = 0.5;

  curr_system = system;

  /* Take at maximum 1000 steps.  The algorithm should quit before
     then, when it converges on a minimum. */
  for (step_num = 0 ; step_num < 1000 ; step_num ++)
    {
      /* Find the energy of the current system */
      curr_energy = eval_func(curr_system);

      /* Take a random step and find the energy of the resulting
         system */
      new_system = step_func(curr_system);
      new_energy = eval_func(new_system);

      /* Determine whether or not to switch to this new system.  This
	 is the core of the annealing simulation and is called the
	 Metropolis algorithm.  The gist of the algorithm is that we
	 always take a downhill step, and sometimes take an uphill
	 step, depending on the system's temperature. */
      if (new_energy < curr_energy)
	{
	  /* Always choose a system with lower energy */
	  curr_system = new_system;
	  curr_energy = new_energy;
	}
      else if (new_energy > curr_energy)
	{
	  double r;

	  r = (double) rand() / (double) RAND_MAX;

	  printf("Guy: %g\n",
		 exp ( - (new_energy - curr_energy) / temperature));

#if 0	  
	  if ( r < exp ( - (new_energy - curr_energy) / temperature) )
	    {
	      printf("  Uphill!\n");
	      curr_system = new_system;
	      curr_energy = new_energy;
	    }
#endif
	}

      printf("Temp: %g  Energy: %ld System: %g\n",
	     temperature, curr_energy, *(double *)curr_system);

      temperature *= 0.9;
    }

  return curr_system;

} /* anneal */
       
#endif
