#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "order.h"
#include "seq.h"
#include "step.h"
#include "cost.h"
#include "anneal.h"

#define MAX_UNSUCCESFUL_STEPS 1000

int
main(int argc, char ** argv)
{
  unsigned long curr_energy, new_energy;
  order_t * curr_order, * new_order;
  seq_t * seq;
  int unsuccesful_steps = 0;
  int use_pf;

  if (argc < 3)
    {
      fprintf(stderr, "Usage: %s <executable> <seq file> [-pf]\n", *argv);
      exit(1);
    }

  use_pf = 0;
  if (argc > 3)
    use_pf = 1;

  srand(getpid());

  fprintf(stderr, "Loading executable...\n");
  curr_order = order_load(argv[1]);
  if (curr_order == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[1]);
      exit(1);
    }

  fprintf(stderr, "Loading sequence graph...\n");
  seq = seq_load(argv[2], curr_order);
  if (seq == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[2]);
      exit(1);
    }

  fprintf(stderr, "Filtering out uncalled functions...\n");
  new_order = order_filter_seq(curr_order, seq);
  order_destroy(curr_order);
  curr_order = new_order;

  fprintf(stderr, "Reloading sequence graph...\n");
  seq_destroy(seq, curr_order);
  seq = seq_load(argv[2], curr_order);

#if 0
  low_pc = curr_order->func[0]->vma;
  order_span_randomize(curr_order, 0, curr_order->num_funcs - 1);
  order_fix_vmas(curr_order, low_pc);
#endif

  fprintf(stderr, "Initial energy: (seq: %ld) (pf: %ld)\n",
	 order_energy_seq(curr_order, seq),
	 order_energy_pf(curr_order, seq));

  fprintf(stderr, "Diving to the nearest local minimum...\n");

  if (use_pf)
    curr_energy = order_energy_pf(curr_order, seq);
  else
    curr_energy = order_energy_seq(curr_order, seq);

  while (1)
    {
      new_order = order_copy(curr_order);
      order_step (new_order);

      if (use_pf)
	curr_energy = order_energy_pf(curr_order, seq);
      else
	new_energy = order_energy_seq(new_order, seq);

      if (new_energy < curr_energy)
	{
	  /* Take a downhill step */
	  fprintf(stderr, "Stepping to (seq: %ld) (pf: %ld)\n",
		 order_energy_seq(new_order, seq),
		 order_energy_pf(new_order, seq));
	  order_destroy(curr_order);
	  curr_order = new_order;
	  curr_energy = new_energy;
	  unsuccesful_steps = 0;
	}
      else
	{
	  order_destroy (new_order);

	  unsuccesful_steps ++;
	  if (unsuccesful_steps > MAX_UNSUCCESFUL_STEPS)
	    {
	      fprintf(stderr, "Examined %d unsuccesful steps.  Local minimum "
		     "reached.\n", unsuccesful_steps);
	      order_print(curr_order);
	      return 0;
	    }
	}
    }

  return 0;
} /* main */
