#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "pages.h"

pages_t *
pages_new(unsigned long start_addr, unsigned long end_addr, int page_size)
{
  pages_t * pages;

  pages = (pages_t *) malloc(sizeof(pages_t));
  if (pages == NULL)
    {
      fprintf(stderr, "pages_init: Out of memory\n");
      return NULL;
    }

  pages->page_size = page_size;
  pages->start_addr = start_addr;

  pages->num_pages = (int) ceil((double) (end_addr - start_addr) /
			  (double) page_size);
  if (pages->num_pages == 0)
    pages->num_pages = 1;

  pages->page = (char *) malloc(pages->num_pages * sizeof(char));
  if (pages->page == NULL)
    {
      fprintf(stderr, "pages_init: Out of memory\n");
      return NULL;
    }

  memset(pages->page, 0, pages->num_pages * sizeof(char));

  return pages;
} /* pages_new */

void
pages_destroy(pages_t * pages)
{
  free(pages->page);
  free(pages);
} /* pages_destroy */

void
pages_clear(pages_t * pages)
{
  memset(pages->page, 0, pages->num_pages * sizeof(char));
} /* pages_clear */

unsigned char
page_inc(pages_t * pages, unsigned long addr, int step)
{
  unsigned int page_num;

  page_num = (addr - pages->start_addr) / pages->page_size;

  if ((((int) pages->page[page_num]) + step) > 255)
    {
      pages->page[page_num] = 255;
      return pages->page[page_num];
    }

  if ((((int) pages->page[page_num]) + step) < 0)
    {
      pages->page[page_num] = 0;
      return pages->page[page_num];
    }

  pages->page[page_num] += step;

  return pages->page[page_num];
} /* page_inc */

void
page_span_inc(pages_t * pages, unsigned long addr, unsigned long size,
	      int step)
{
  unsigned long curr_addr;
  unsigned long end_addr;

  end_addr = addr + size;

  /* Touch the pages on each end of the span, then touch all the ones
     in between. */
  page_inc (pages, addr, step);

  /* If the whole span is contained on one page, we're done */
  if ((end_addr / pages->page_size) == (addr / pages->page_size))
    return;

  /* Touch the page on the end */
  page_inc (pages, end_addr, step);

  /* Now touch all in-between pages */
  for (curr_addr = (addr + pages->page_size);
       (curr_addr / pages->page_size) != (end_addr / pages->page_size);
       curr_addr += pages->page_size)
    {
      page_inc (pages, curr_addr, step);
    }
} /* page_span_inc */

