#include <stdio.h>
#include "order.h"
#include "seq.h"

int
main(int argc, char ** argv)
{
  order_t * order;
  seq_t * seq;
  unsigned long used, total, num_funcs;
  list_t * curr;

  if (argc < 3)
    {
      fprintf(stderr, "Usage: %s <executable> <sequence graph>\n", *argv);
      exit(1);
    }
  
  printf("Loading executable...\n");
  order = order_load(argv[1]);
  if (order == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[1]);
      exit(1);
    }

  printf("Loading sequence graph...\n");
  seq = seq_load(argv[2], order);
  if (seq == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[2]);
      exit(1);
    }

  total = (order->func[order->num_funcs - 1]->vma +
	   order->func[order->num_funcs - 1]->size) -
    order->func[0]->vma;

  used = 0;
  num_funcs = 0;
  for (curr = seq->graph->nodes ; curr != NULL ; curr = curr->next)
    {
      node_t * node;
      func_t * func;

      node = (node_t *)curr->data;
      func = order->func[order->func_idx[(int) node->data]];

      used += func->size;
      num_funcs ++;
    }

  printf("%d / %d functions were called\n", num_funcs, order->num_funcs);
  printf("%d / %d of the code in the executable was used\n",
	 used, total);
  return 0;
} /* main */
