/* Print out the energy of a program, given the executable and sequence
   file */
#include <stdio.h>
#include <stdlib.h>
#include "order.h"
#include "seq.h"
#include "cost.h"

int
main(int argc, char ** argv)
{
  unsigned long min_cost;
  list_t * curr;
  order_t * order;
  seq_t * seq;

  if (argc < 3)
    {
      fprintf(stderr, "Usage: %s <executable> <sequence graph>\n", *argv);
      exit(1);
    }

  printf("Loading order of functions...\n");
  order = order_load(argv[1]);

  printf("Loading sequence graph...\n");
  seq = seq_load(argv[2], order);

  printf("order_energy_seq: %ld\n", order_energy_seq(order, seq));
  printf("order_energy_pf: %ld\n", order_energy_pf(order, seq));

  min_cost = 0;
  for (curr = seq->graph->arcs; curr != NULL ; curr = curr->next)
    {
      arc_t * arc = (arc_t *) curr->data;

      min_cost += arc->w;
    }

  printf("minimum (theoretical) seq energy: %ld\n", min_cost);

  return 0;
} /* main */
