/* The functions which take a random step in an ordering */

#include <stdlib.h>
#include <stdio.h>

#include "order.h"
#include "step.h"

void
order_step(order_t * order)
{
  int type;

  /* Choose a type of step */
  type = rand() % 4;

  switch(type)
    {
    case 0:
      order_step_randomize(order);
      return;
    case 1:
      order_step_swap(order);
      return;
    case 2:
      order_step_swap2 (order);
      return;
    case 3:
      order_step_move (order);
      return;
    default:
      fprintf(stderr, "order_step: Internal error\n");
      exit(1);
    }
} /* order_step */

/*
 * Randomize the order of the funtions in a random section of the
 * path.
 */
void
order_step_randomize(order_t * order)
{
  int start_func, end_func, num_funcs, i;
  unsigned long low_pc;

  /* Save this for later */
  low_pc = order->func[0]->vma;

  /* First choose the extents of the section of the sequence which is
     going to be randomized. */
  num_funcs = rand() % (order->num_funcs / 5);
  start_func = rand() % order->num_funcs;
  end_func = start_func + num_funcs - 1;
  if (end_func > (order->num_funcs - 1))
    end_func = order->num_funcs - 1;

  printf("\tRandomizing order of functions %d - %d\n", start_func,
	 end_func);

  order_span_randomize(order, start_func, end_func);

  order_fix_vmas(order, low_pc);
} /* order_step_randomize */

/*
 * Move a random block of functions to a random place in the executable
 */
void
order_step_move (order_t * order)
{
  int start_func, end_func, num_funcs, pre_func, i;
  func_t ** temp_func_list;
  unsigned long low_pc;

  /* Save this for later */
  low_pc = order->func[0]->vma;

  /* First choose the extents of the section of the sequence which is
     going to be moved. */
  num_funcs = rand() % (order->num_funcs / 5);
  start_func = rand() % order->num_funcs;
  end_func = start_func + num_funcs - 1;
  if (end_func > (order->num_funcs - 1))
    end_func = order->num_funcs - 1;
  num_funcs = end_func - start_func + 1;

  /* Save all these functions into temp_func_list */
  temp_func_list = (func_t **) malloc(sizeof(func_t *) * num_funcs);
  if (temp_func_list == NULL)
    {
      fprintf(stderr, "order_step_move: Out of memory\n");
      exit(1);
    }
  for (i = start_func ; i <= end_func ; i ++)
    temp_func_list[i - start_func] = order->func[i];

  /* Slide all of the functions after the selected block down where
     this block used to be. */
  for (i = end_func + 1 ; i < order->num_funcs ; i ++)
    order->func[i - num_funcs] = order->func[i];

  /* Now choose a function after which all these functions will be placed. */
  pre_func = rand() % (order->num_funcs - num_funcs);

  printf("\tMoving funcs %d - %d to %d\n", start_func, end_func, pre_func);

  /* Move all of the functions after pre_func down to make room for the
     block. */
  for (i = order->num_funcs - num_funcs - 1; i > pre_func; i--)
    order->func[i + num_funcs] = order->func[i];

  /* Now drop the block into place */
  for (i = 0 ; i < num_funcs ; i++)
    order->func[pre_func + 1 + i] = temp_func_list[i];
      
  /* Fix up the func_idx array */
  for (i = 0 ; i < order->num_funcs ; i++)
    order->func_idx[order->func[i]->idx] = i;

  order_fix_vmas(order, low_pc);
} /* order_step_move */

/*
 * Swap the positions of two functions in the list
 */
void
order_step_swap(order_t * order)
{
  unsigned int func1, func2;
  unsigned int idx_a, idx_b;
  unsigned long low_pc;
  func_t * temp;
  int i;

  low_pc = order->func[0]->vma;

  /* Choose two functions */
  func1 = rand() % order->num_funcs;
  func2 = rand() % order->num_funcs;

  printf("\tSwapping %d and %d\n", func1, func2);

  /* Swap them */
  idx_a = order->func[func1]->idx;
  idx_b = order->func[func2]->idx;
  order->func_idx[idx_a] = func2;
  order->func_idx[idx_b] = func1;

  temp = order->func[func1];
  order->func[func1] = order->func[func2];
  order->func[func2] = temp;

  order_fix_vmas(order, low_pc);
} /* order_step_swap */


/*
 * Swap the positions of N (5) functions in the list
 */
void
order_step_swap2(order_t * order)
{
  unsigned int func1, func2;
  unsigned int idx_a, idx_b;
  unsigned long low_pc;
  func_t * temp;
  int i;

  low_pc = order->func[0]->vma;

  printf("\tPerforming 10 swaps...\n");
  for (i = 0 ; i < 10 ; i ++)
    {
      /* Choose two functions */
      func1 = rand() % order->num_funcs;
      func2 = rand() % order->num_funcs;

      /* Swap them */
      idx_a = order->func[func1]->idx;
      idx_b = order->func[func2]->idx;
      order->func_idx[idx_a] = func2;
      order->func_idx[idx_b] = func1;

      temp = order->func[func1];
      order->func[func1] = order->func[func2];
      order->func[func2] = temp;
    }

  order_fix_vmas(order, low_pc);
} /* order_step_swap2 */


