#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "order.h"
#include "seq.h"
#include "step.h"
#include "cost.h"

int
main(int argc, char ** argv)
{
  int i;
  unsigned long energy, new_energy;
  order_t * order, * new;
  seq_t * seq;

  if (argc < 3)
    {
      fprintf(stderr, "Usage: %s <executable> <seq file>\n", *argv);
      exit(1);
    }

  printf("Loading order of functions...\n");
  order = order_load(argv[1]);

  printf("Loading sequence graph...\n");
  seq = seq_load(argv[2], order);

  new = (order_t *) malloc(sizeof(order_t));
  if (new == NULL)
    {
      fprintf(stderr, "Out of memory\n");
      exit(1);
    }

  srand(getpid());

#if 0
  printf("Original order:\n");
  order_print(order);
  printf("\n\nNew order:\n");
  order_print(new);
  printf("\n\n");
#endif

  energy = order_energy_seq(order, seq);

  i = 0;
  while (1)
    {
      i++;
      if (i > 100)
	exit(0);
	
      new = order_copy(order);
      order_step(new);

      new_energy = order_energy_seq(new, seq);

      if (new_energy < energy)
	{
	  printf("Accepting new ordering of energy %ld\n", new_energy);
	  order_destroy(order);
	  order = new;
	  energy = new_energy;
	}
      else
	{
	  order_destroy(new);
	  /*	  printf("Rejecting ordering of energy %ld\n", new_energy);*/
	}
    }

  return 0;
} /* main */
