#include <stdio.h>
#include "graph.h"
#include "group.h"
#include "order.h"
#include "seq.h"

int
main(int argc, char ** argv)
{
  order_t * order;
  seq_t * seq;

  if (argc < 3)
    {
      fprintf(stderr, "Usage: %s <executable> <sequence graph>\n", *argv);
      exit(1);
    }

  fprintf(stderr, "Loading executable...\n");
  order = order_load(argv[1]);
  if (order == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[1]);
      exit(1);
    }

  fprintf(stderr, "Loading sequence graph...\n");
  seq = seq_load(argv[2], order);
  if (seq == NULL)
    {
      fprintf(stderr, "Error loading %s\n", argv[2]);
      exit(1);
    }

  /* Change the node private data pointers in the graph to point
     to the function names, not the function indices */
  seq_resolve_names(seq, order);

  fprintf(stderr, "Running node grouping algorithm...\n");
  graph_group(seq->graph);

  /* The graph in seq->graph now contains only one node, which is
     a concatenation of all of the function indices from the graph,
     in the proper order. */
  if (seq->graph->nodes->next != NULL)
    fprintf(stderr, "Error: More than one node remains!\n");
  printf("Remaining node:\n %s\n", ((node_t *)seq->graph->nodes->data)->data);
  
 
  return 0;
} /* main */







