
#ifndef __GRAPH_H__
#define __GRAPH_H__

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/* Data types */
typedef unsigned long weight_t;

typedef struct list_struct {
  void * data;
  struct list_struct * next;
} list_t;

typedef struct node_struct {
  /* The list of arcs connecting to this node */
  struct list_struct * arcs;
  weight_t w;

  void * data;
} node_t;

typedef struct arc_struct {
  /* The two nodes which make up this arc */
  node_t * n1;
  node_t * n2;
  weight_t w;

  void * data;
} arc_t;

typedef struct graph_struct {
  list_t * arcs;
  list_t * nodes;
} graph_t;

/* Prototypes */
list_t * list_new(void * data);
void list_delete(list_t * list);
int list_element_present_p(list_t * list, void * element);
list_t * list_add(list_t * list, void * data);
list_t * list_remove_element(list_t * list, void * element);

node_t * node_new (weight_t w, void * data);

arc_t * arc_new (node_t * n1, node_t * n2, weight_t w, void * data);
void arc_delete(arc_t * arc);

graph_t * graph_new(void);
void graph_node_add(graph_t * graph, node_t * node);
int graph_node_remove(graph_t * graph, node_t * node);
void graph_arc_add(graph_t * graph, arc_t * arc);
void graph_arc_new(graph_t * graph, node_t * n1, node_t * n2, weight_t w,
		   void * data);
void graph_arc_sort(graph_t * graph);
arc_t * graph_arc_get(graph_t * graph, node_t * n1, node_t * n2);
void graph_arc_delete(graph_t * graph, arc_t * arc);
void graph_arc_inc(graph_t * graph, node_t * n1, node_t * n2, weight_t step);
void graph_arc_set(graph_t * graph, node_t * n1, node_t * n2, int weight);
void graph_print(graph_t * graph);
void graph_destroy(graph_t * graph);

#endif /* __GRAPH_H__ */
