#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "graph.h"
#include "group.h"

static void
group_add_arcs(graph_t * graph, node_t * dest_node, node_t * source_node,
	 node_t * node)
{
  list_t * curr;

  for (curr = source_node->arcs ; curr != NULL ; curr = curr->next)
    {
      arc_t * curr_arc;
      node_t * other;
      weight_t w;

      curr_arc = (arc_t *) curr->data;

      /* Find the 'other' node */
      if (curr_arc->n1 != source_node)
	other = curr_arc->n1;
      else
	other = curr_arc->n2;

      w = curr_arc->w;

      if (other != node)
	{
	  arc_t * n2_arc;
	  /* Now, find out if 'node' also had an arc with 'other,' and
	     if so, add the weights. */
	  n2_arc = graph_arc_get(graph, other, node);
	  if (n2_arc != NULL)
	    {
	      w += n2_arc->w;
	      graph_arc_delete (graph, n2_arc);
	    }

	  /* Now create a new arc for this new node */
	  graph_arc_new(graph, dest_node, other, w, NULL);
	}
    }

} /* group_add_arcs */

void
graph_group(graph_t * graph)
{
  arc_t * arc;
  node_t * new_node;
  char * name;

  /* First print the graph */
  graph_print(graph);

  /* While there are still arcs... */
  while (graph->arcs != NULL)
    {
      node_t * node1, * node2;
      /* Grab the highest arc in the graph */
      graph_arc_sort(graph);
      arc = (arc_t *) graph->arcs->data;

      /* Create a new node which is the combination of the two nodes
	 which made up the highest arc */
      name = malloc(strlen((char *)arc->n1->data) +
		    strlen((char *)arc->n2->data) +
		    2);
      strcpy(name, (char *)arc->n1->data);
      strcat(name, "\n");
      strcat(name, (char *)arc->n2->data);

      new_node = node_new(0, name);
      graph_node_add(graph, new_node);

      /* Add arcs for this new node.  This means that, for each of
	 the two nodes that made up the highest arc, we examine every
	 arc which connected to that node. FIXME */
      group_add_arcs(graph, new_node, arc->n1, arc->n2);
      group_add_arcs(graph, new_node, arc->n2, arc->n1);

      /* Now delete the old nodes */
      node1 = arc->n1;
      node2 = arc->n2;
      graph_arc_delete(graph, arc);
      graph_node_remove(graph, node1);
      graph_node_remove(graph, node2);
      free(node1);
      free(node2);

      printf("Completed a grouping...\n");
    }

  graph_print(graph);
  
} /* graph_group */


